=== modified file 'NEWS'
--- NEWS	2006-08-23 23:21:12 +0000
+++ NEWS	2006-08-25 14:36:23 +0000
@@ -23,6 +23,8 @@
 
   BUG FIXES:
 
+    * ``bzr uncommit`` preserves pending merges. (John Arbash Meinel, #57660)
+
     * Active FTP transport now works as intended. (ghozzy, #56472)
 
     * Really fix mutter() so that it won't ever raise a UnicodeError.

=== modified file 'bzrlib/tests/blackbox/test_uncommit.py'
--- bzrlib/tests/blackbox/test_uncommit.py	2006-07-11 21:09:40 +0000
+++ bzrlib/tests/blackbox/test_uncommit.py	2006-08-25 14:36:23 +0000
@@ -25,6 +25,7 @@
 
 
 class TestUncommit(TestCaseWithTransport):
+
     def create_simple_tree(self):
         wt = self.make_branch_and_tree('tree')
         self.build_tree(['tree/a', 'tree/b', 'tree/c'])
@@ -143,6 +144,19 @@
         self.assertEqual('a2', wt.last_revision())
         self.assertEqual(['b4'], wt.pending_merges())
 
+    def test_uncommit_pending_merge(self):
+        wt = self.create_simple_tree()
+        tree2 = wt.bzrdir.sprout('tree2').open_workingtree()
+        tree2.commit('unchanged', rev_id='b3')
+
+        wt.branch.fetch(tree2.branch)
+        wt.set_pending_merges(['b3'])
+
+        os.chdir('tree')
+        out, err = self.run_bzr('uncommit', '--force')
+        self.assertEqual('a1', wt.last_revision())
+        self.assertEqual(['b3'], wt.pending_merges())
+
     def test_uncommit_multiple_merge(self):
         wt = self.create_simple_tree()
 
@@ -167,6 +181,29 @@
         self.assertEqual('a2', wt.last_revision())
         self.assertEqual(['b3', 'b4'], wt.pending_merges())
 
+    def test_uncommit_merge_plus_pending(self):
+        wt = self.create_simple_tree()
+
+        tree2 = wt.bzrdir.sprout('tree2').open_workingtree()
+
+        tree2.commit('unchanged', rev_id='b3')
+        wt.branch.fetch(tree2.branch)
+        wt.set_pending_merges(['b3'])
+        wt.commit('merge b3', rev_id='a3')
+
+        tree2.commit('unchanged', rev_id='b4')
+        wt.branch.fetch(tree2.branch)
+        wt.set_pending_merges(['b4'])
+
+        self.assertEqual('a3', wt.last_revision())
+        self.assertEqual(['b4'], wt.pending_merges())
+
+        os.chdir('tree')
+        out, err = self.run_bzr('uncommit', '--force', '-r', '2')
+
+        self.assertEqual('a2', wt.last_revision())
+        self.assertEqual(['b3', 'b4'], wt.pending_merges())
+
     def test_uncommit_octopus_merge(self):
         # Check that uncommit keeps the pending merges in the same order
         wt = self.create_simple_tree()

=== modified file 'bzrlib/uncommit.py'
--- bzrlib/uncommit.py	2006-08-08 23:19:29 +0000
+++ bzrlib/uncommit.py	2006-08-25 14:36:23 +0000
@@ -18,14 +18,9 @@
 """
 
 import os
+
 from bzrlib.errors import BoundBranchOutOfDate
 
-def test_remove(filename):
-    if os.path.exists(filename):
-        os.remove(filename)
-    else:
-        print '* file does not exist: %r' % filename
-
 
 def uncommit(branch, dry_run=False, verbose=False, revno=None, tree=None):
     """Remove the last revision from the supplied branch.
@@ -44,6 +39,8 @@
         unlockable.append(branch)
 
         pending_merges = []
+        if tree is not None:
+            pending_merges = tree.pending_merges()
 
         master = branch.get_master_branch()
         if master is not None:

